@@ -56,7 +56,7 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
                     stdout=subprocess.PIPE)
             stdout, _ = process.communicate()
 
-            for curline in stdout.split('\n'):
+            for curline in stdout.decode('utf-8').split('\n'):
                 items = curline.strip().split('|')
                 if items[0] in currentjobs and items[1] == 'RUN':
                     jobstart = parse(items[2], default=datetime.now(tzlocal()))
@@ -75,7 +75,7 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
             combinedEnv.update(os.environ)
             process = subprocess.Popen(subLine, stdout=subprocess.PIPE,
                                        env=combinedEnv)
-            line = process.stdout.readline()
+            line = process.stdout.readline().decode('utf-8')
             logger.debug("BSUB: " + line)
             result = int(line.strip().split()[1].strip('<>'))
             logger.debug("Got the job id: {}".format(result))
@@ -110,8 +110,8 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
                 elif "Exited with exit code" in line:
                     exit = int(line[line.find("Exited with exit code ")+22:]
                                .split('.')[0])
-                    logger.error("bjobs detected job exit code: "
-                                 "{}".format(exit))
+                    logger.error("bjobs detected job exit code "
+                                 "{} for job {}".format(exit, job))
                     return exit
                 elif "Completed <exit>" in line:
                     logger.error("bjobs detected job failed for job: "
@@ -173,10 +173,12 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
             else:
                 bsubMem = []
             bsubCpu = [] if cpu is None else ['-n', str(math.ceil(cpu))]
-            bsubline = ["bsub", "-cwd", ".", "-o", "/dev/null",
-                        "-e", "/dev/null", "-J", "toil_job_{}".format(jobID)]
+            bsubline = ["bsub", "-cwd", ".", "-J", "toil_job_{}".format(jobID)]
             bsubline.extend(bsubMem)
             bsubline.extend(bsubCpu)
+            stdoutfile = self.boss.formatStdOutErrPath(jobID, 'lsf', '%J', 'std_output')
+            stderrfile = self.boss.formatStdOutErrPath(jobID, 'lsf', '%J', 'std_error')
+            bsubline.extend(['-o', stdoutfile, '-e', stderrfile])
             lsfArgs = os.getenv('TOIL_LSF_ARGS')
             if lsfArgs:
                 bsubline.extend(lsfArgs.split())
@@ -191,7 +193,7 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
         p = subprocess.Popen(["lshosts"], stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
 
-        line = p.stdout.readline()
+        line = p.stdout.readline().decode('utf-8')
         items = line.strip().split()
         num_columns = len(items)
         cpu_index = None
@@ -206,7 +208,7 @@ class LSFBatchSystem(AbstractGridEngineBatchSystem):
                 RuntimeError("lshosts command does not return ncpus or maxmem "
                              "columns")
 
-        # p.stdout.readline()
+        # p.stdout.readline().decode('utf-8')
 
         maxCPU = 0
         maxMEM = MemoryString("0")
