@@ -39,6 +39,7 @@ from six.moves.urllib import parse as urlparse
 import six
 
 from schema_salad import validate
+from schema_salad.schema import Names
 import schema_salad.ref_resolver
 
 import cwltool.errors
@@ -58,7 +59,7 @@ from cwltool.pathmapper import (PathMapper, adjustDirObjs, adjustFileObjs,
                                 get_listing, MapperEnt, visit_class,
                                 normalizeFilesDirs)
 from cwltool.process import (shortname, fill_in_defaults, compute_checksums,
-                             collectFilesAndDirs, add_sizes)
+                             add_sizes)
 from cwltool.secrets import SecretStore
 from cwltool.software_requirements import (
     DependenciesConfiguration, get_container_from_software_requirements)
@@ -304,7 +305,7 @@ def toil_get_file(file_store, index, existing, file_store_id):
     """Get path to input file from Toil jobstore."""
 
     if not file_store_id.startswith("toilfs:"):
-        return schema_salad.ref_resolver.file_uri(file_store_id)
+        return file_store.jobStore.getPublicUrl(file_store.jobStore.importFile(file_store_id))
     src_path = file_store.readGlobalFile(file_store_id[7:])
     index[src_path] = file_store_id
     existing[file_store_id] = src_path
@@ -354,7 +355,7 @@ def uploadFile(uploadfunc, fileindex, existing, uf, skip_broken=False):
         return
     if not uf["location"] and uf["path"]:
         uf["location"] = schema_salad.ref_resolver.file_uri(uf["path"])
-    if not os.path.isfile(uf["location"][7:]):
+    if uf["location"].startswith("file://") and not os.path.isfile(uf["location"][7:]):
         if skip_broken:
             return
         else:
@@ -389,8 +390,21 @@ def toilStageFiles(file_store, cwljob, outdir, index, existing, export,
     """Copy input files out of the global file store and update location and
     path."""
 
-    jobfiles = []  # type: List[Dict[Text, Any]]
-    collectFilesAndDirs(cwljob, jobfiles)
+    def _collectDirEntries(obj):
+    # type: (Union[Dict[Text, Any], List[Dict[Text, Any]]]) -> Iterator[Dict[Text, Any]]
+        if isinstance(obj, dict):
+            if obj.get("class") in ("File", "Directory"):
+                yield obj
+            else:
+                for sub_obj in obj.values():
+                    for dir_entry in _collectDirEntries(sub_obj):
+                        yield dir_entry
+        elif isinstance(obj, list):
+            for sub_obj in obj:
+                for dir_entry in _collectDirEntries(sub_obj):
+                    yield dir_entry
+
+    jobfiles = list(_collectDirEntries(cwljob))
     pm = ToilPathMapper(
         jobfiles, "", outdir, separateDirs=False, stage_listing=True)
     for f, p in pm.items():
@@ -497,21 +511,45 @@ class CWLJob(Job):
         if runtime_context.builder:
             builder = runtime_context.builder
         else:
-            builder = cwltool.builder.Builder(cwljob)
-            builder.requirements = self.cwltool.requirements
-            builder.outdir = None
-            builder.tmpdir = None
-            builder.timeout = runtime_context.eval_timeout
-            builder.resources = {}
+            builder = cwltool.builder.Builder(
+                job=cwljob,
+                files=[],
+                bindings=[],
+                schemaDefs={},
+                names=Names(),
+                requirements=self.cwltool.requirements,
+                hints=[],
+                resources={},
+                mutation_manager=None,
+                formatgraph=None,
+                make_fs_access=runtime_context.make_fs_access,
+                fs_access=runtime_context.make_fs_access(''),
+                job_script_provider=None,
+                timeout=runtime_context.eval_timeout,
+                debug=False,
+                js_console=False,
+                force_docker_pull=False,
+                loadListing=u'',
+                outdir=u'',
+                tmpdir=u'',
+                stagedir=u''
+            )
         req = tool.evalResources(builder, runtime_context)
         # pass the default of None if basecommand is empty
         unitName = self.cwltool.tool.get("baseCommand", None)
         if isinstance(unitName, (MutableSequence, tuple)):
             unitName = ' '.join(unitName)
+
+        try:
+            displayName = str(self.cwltool.tool['id'])
+        except KeyError:
+            displayName = None
+
         super(CWLJob, self).__init__(
             cores=req["cores"], memory=int(req["ram"]*(2**20)),
             disk=int((req["tmpdirSize"]*(2**20))+(req["outdirSize"]*(2**20))),
-            unitName=unitName)
+            unitName=unitName,
+            displayName=displayName)
 
         self.cwljob = cwljob
         try:
@@ -530,7 +568,8 @@ class CWLJob(Job):
         fill_in_defaults(
             self.step_inputs, cwljob,
             self.runtime_context.make_fs_access(""))
-        for inp_id in cwljob.keys():
+        immobile_cwljob_dict = copy.deepcopy(cwljob)
+        for inp_id in immobile_cwljob_dict.keys():
             found = False
             for field in self.cwltool.inputs_record_schema['fields']:
                 if field['name'] == inp_id:
@@ -1017,6 +1056,7 @@ def main(args=None, stdout=sys.stdout):
     parser.add_argument("--basedir", type=str)
     parser.add_argument("--outdir", type=str, default=os.getcwd())
     parser.add_argument("--version", action='version', version=baseVersion)
+
     dockergroup = parser.add_mutually_exclusive_group()
     dockergroup.add_argument(
         "--user-space-docker-cmd",
@@ -1031,27 +1071,36 @@ def main(args=None, stdout=sys.stdout):
         "--no-container", action="store_true", help="Do not execute jobs in a "
         "Docker container, even when `DockerRequirement` "
         "is specified under `hints`.")
+
     parser.add_argument(
         "--preserve-environment", type=str, nargs='+',
         help="Preserve specified environment variables when running"
         " CommandLineTools", metavar=("VAR1 VAR2"), default=("PATH",),
         dest="preserve_environment")
+    parser.add_argument(
+        "--preserve-entire-environment", action="store_true",
+        help="Preserve all environment variable when running "
+             "CommandLineTools.",
+        default=False, dest="preserve_entire_environment")
     parser.add_argument(
         "--destBucket", type=str,
         help="Specify a cloud bucket endpoint for output files.")
     parser.add_argument(
         "--beta-dependency-resolvers-configuration", default=None)
-    parser.add_argument("--beta-dependencies-directory", default=None)
+    parser.add_argument(
+        "--beta-dependencies-directory", default=None)
     parser.add_argument(
         "--beta-use-biocontainers", default=None, action="store_true")
     parser.add_argument(
         "--beta-conda-dependencies", default=None, action="store_true")
-    parser.add_argument("--tmpdir-prefix", type=Text,
-                        help="Path prefix for temporary directories",
-                        default="tmp")
-    parser.add_argument("--tmp-outdir-prefix", type=Text,
-                        help="Path prefix for intermediate output directories",
-                        default="tmp")
+    parser.add_argument(
+        "--tmpdir-prefix", type=Text,
+        help="Path prefix for temporary directories",
+        default="tmp")
+    parser.add_argument(
+        "--tmp-outdir-prefix", type=Text,
+        help="Path prefix for intermediate output directories",
+        default="tmp")
     parser.add_argument(
         "--force-docker-pull", action="store_true", default=False,
         dest="force_docker_pull",
@@ -1059,23 +1108,44 @@ def main(args=None, stdout=sys.stdout):
     parser.add_argument(
         "--no-match-user", action="store_true", default=False,
         help="Disable passing the current uid to `docker run --user`")
+    parser.add_argument(
+        "--no-read-only", action="store_true", default=False,
+        help="Do not set root directory in the container as read-only")
 
+    if args is None:
+        args = sys.argv[1:]
+
+    # Problem: we want to keep our job store somewhere auto-generated based on
+    # our options, unless overridden by... an option. So we will need to parse
+    # options twice, because we need to feed the parser the job store.
+    
+    # Propose a local workdir, probably under /tmp.
     # mkdtemp actually creates the directory, but
     # toil requires that the directory not exist,
+    # since it is going to be our jobstore,
     # so make it and delete it and allow
     # toil to create it again (!)
     workdir = tempfile.mkdtemp()
     os.rmdir(workdir)
 
-    if args is None:
-        args = sys.argv[1:]
-
-    # we use workdir as jobStore:
+    # we use workdir as default default jobStore:
     options = parser.parse_args([workdir] + args)
 
-    # if tmpdir_prefix is not the default value, set workDir too
+    # if tmpdir_prefix is not the default value, set workDir if unset, and move
+    # workdir and the job store under it
     if options.tmpdir_prefix != 'tmp':
-        options.workDir = options.tmpdir_prefix
+        workdir = tempfile.mkdtemp(dir=options.tmpdir_prefix)
+        os.rmdir(workdir)
+        # Re-parse arguments with the new default jobstore under the temp dir.
+        # It still might be overridden by a --jobStore option
+        options = parser.parse_args([workdir] + args)
+        if options.workDir is None:
+            # We need to override workDir because by default Toil will pick
+            # somewhere under the system temp directory if unset, ignoring
+            # --tmpdir-prefix.
+            #
+            # If set, workDir needs to exist, so we directly use the prefix
+            options.workDir = options.tmpdir_prefix
 
     if options.provisioner and not options.jobStore:
         raise NoSuchJobStoreException(
@@ -1134,23 +1204,11 @@ def main(args=None, stdout=sys.stdout):
                 cwltool.main.load_job_order(
                     options, sys.stdin, loading_context.fetcher_constructor,
                     loading_context.overrides_list, tool_file_uri)
-            document_loader, workflowobj, uri = \
-                cwltool.load_tool.fetch_document(
-                    uri, loading_context.resolver,
-                    loading_context.fetcher_constructor)
-            document_loader, avsc_names, processobj, metadata, uri = \
-                cwltool.load_tool.validate_document(
-                    document_loader, workflowobj, uri,
-                    loading_context.enable_dev, loading_context.strict, False,
-                    loading_context.fetcher_constructor, False,
-                    loading_context.overrides_list,
-                    do_validate=loading_context.do_validate)
-            loading_context.overrides_list.extend(
-                metadata.get("cwltool:overrides", []))
+            loading_context, workflowobj, uri = cwltool.load_tool.fetch_document(uri, loading_context)
+            loading_context, uri = cwltool.load_tool.resolve_and_validate_document(loading_context, workflowobj, uri)
+            loading_context.overrides_list.extend(loading_context.metadata.get("cwltool:overrides", []))
             try:
-                tool = cwltool.load_tool.make_tool(
-                    document_loader, avsc_names, metadata, uri,
-                    loading_context)
+                tool = cwltool.load_tool.make_tool(uri, loading_context)
             except cwltool.process.UnsupportedRequirement as err:
                 logging.error(err)
                 return 33
@@ -1182,20 +1240,17 @@ def main(args=None, stdout=sys.stdout):
 
             for inp in tool.tool["inputs"]:
                 def set_secondary(fileobj):
-                    if isinstance(fileobj, Mapping) \
-                            and fileobj.get("class") == "File":
+                    if isinstance(fileobj, Mapping) and fileobj.get("class") == "File":
                         if "secondaryFiles" not in fileobj:
-                            fileobj["secondaryFiles"] = [
-                                {"location": cwltool.builder.substitute(
-                                    fileobj["location"], sf), "class": "File"}
-                                for sf in inp["secondaryFiles"]]
+                            fileobj["secondaryFiles"] = [{"location": cwltool.builder.substitute(fileobj["location"],
+                                                         sf["pattern"]), "class": "File"}
+                                                         for sf in inp["secondaryFiles"]]
 
                     if isinstance(fileobj, MutableSequence):
                         for entry in fileobj:
                             set_secondary(entry)
 
-                if shortname(inp["id"]) in initialized_job_order \
-                        and inp.get("secondaryFiles"):
+                if shortname(inp["id"]) in initialized_job_order and inp.get("secondaryFiles"):
                     set_secondary(initialized_job_order[shortname(inp["id"])])
 
             import_files(initialized_job_order)
@@ -1209,6 +1264,7 @@ def main(args=None, stdout=sys.stdout):
                 runtime_context.job_script_provider = job_script_provider
                 runtime_context.force_docker_pull = options.force_docker_pull
                 runtime_context.no_match_user = options.no_match_user
+                runtime_context.no_read_only = options.no_read_only
                 (wf1, _) = makeJob(tool, {}, None, runtime_context)
             except cwltool.process.UnsupportedRequirement as err:
                 logging.error(err)
