@@ -389,7 +389,7 @@ class AWSJobStore(AbstractJobStore):
         log.debug("Created %r.", info)
         return info.fileID
 
-    def _importFile(self, otherCls, url, sharedFileName=None):
+    def _importFile(self, otherCls, url, sharedFileName=None, hardlink=False):
         if issubclass(otherCls, AWSJobStore):
             srcKey = self._getKeyForUrl(url, existing=True)
             size = srcKey.size
@@ -489,7 +489,7 @@ class AWSJobStore(AbstractJobStore):
             keyName = url.path[1:]
             bucketName = url.netloc
             bucket = s3.get_bucket(bucketName)
-            key = bucket.get_key(bytes(keyName))
+            key = bucket.get_key(keyName.encode('utf-8'))
             if existing is True:
                 if key is None:
                     raise RuntimeError("Key '%s' does not exist in bucket '%s'." %
@@ -981,12 +981,9 @@ class AWSJobStore(AbstractJobStore):
         def _reservedAttributes(cls):
             return 3 + super(AWSJobStore.FileInfo, cls)._reservedAttributes()
 
-        @classmethod
-        def maxInlinedSize(cls, encrypted):
-            return cls.maxBinarySize() - (encryption.overhead if encrypted else 0)
-
-        def _maxInlinedSize(self):
-            return self.maxInlinedSize(self.encrypted)
+        @staticmethod
+        def maxInlinedSize():
+            return 256
 
         def save(self):
             attributes, numNewContentChunks = self.toItem()
@@ -1021,7 +1018,7 @@ class AWSJobStore(AbstractJobStore):
 
         def upload(self, localFilePath):
             file_size, file_time = fileSizeAndTime(localFilePath)
-            if file_size <= self._maxInlinedSize():
+            if file_size <= self.maxInlinedSize():
                 with open(localFilePath) as f:
                     self.content = f.read()
             else:
@@ -1038,7 +1035,7 @@ class AWSJobStore(AbstractJobStore):
             class MultiPartPipe(WritablePipe):
                 def readFrom(self, readable):
                     buf = readable.read(store.partSize)
-                    if allowInlining and len(buf) <= info._maxInlinedSize():
+                    if allowInlining and len(buf) <= info.maxInlinedSize():
                         info.content = buf
                     else:
                         headers = info._s3EncryptionHeaders()
@@ -1074,7 +1071,7 @@ class AWSJobStore(AbstractJobStore):
             class SinglePartPipe(WritablePipe):
                 def readFrom(self, readable):
                     buf = readable.read()
-                    if allowInlining and len(buf) <= info._maxInlinedSize():
+                    if allowInlining and len(buf) <= info.maxInlinedSize():
                         info.content = buf
                     else:
                         key = store.filesBucket.new_key(key_name=bytes(info.fileID))
@@ -1098,7 +1095,7 @@ class AWSJobStore(AbstractJobStore):
             :param srcKey: The key that will be copied from
             """
             assert srcKey.size is not None
-            if srcKey.size <= self._maxInlinedSize():
+            if srcKey.size <= self.maxInlinedSize():
                 self.content = srcKey.get_contents_as_string()
             else:
                 self.version = copyKeyMultipart(srcBucketName=srcKey.bucket.name,
@@ -1287,8 +1284,10 @@ class AWSJobStore(AbstractJobStore):
                 try:
                     for upload in bucket.list_multipart_uploads():
                         upload.cancel_upload()
+                    keys = list()
                     for key in bucket.list_versions():
-                        bucket.delete_key(key.name, version_id=key.version_id)
+                        keys.append((key.name, key.version_id))
+                    bucket.delete_keys(keys, quiet=True)
                     bucket.delete()
                 except S3ResponseError as e:
                     if e.error_code == 'NoSuchBucket':
