@@ -14,6 +14,7 @@
 import logging
 import os
 from collections import namedtuple
+from difflib import get_close_matches
 from operator import attrgetter
 import datetime
 from toil.lib.misc import std_dev, mean
@@ -26,6 +27,12 @@ logger = logging.getLogger(__name__)
 ZoneTuple = namedtuple('ZoneTuple', ['name', 'price_deviation'])
 
 
+def zoneToRegion(zone):
+    """Get a region (e.g. us-west-2) from a zone (e.g. us-west-1c)."""
+    from toil.lib.context import Context
+    return Context.availability_zone_re.match(zone).group(1)
+
+
 def getSpotZone(spotBid, nodeType, ctx):
     return _getCurrentAWSZone(spotBid, nodeType, ctx)
 
@@ -77,39 +84,33 @@ def choose_spot_zone(zones, bid, spot_history):
     :return: the name of the selected zone
 
     >>> from collections import namedtuple
-    >>> FauxHistory = namedtuple( 'FauxHistory', [ 'price', 'availability_zone' ] )
-    >>> ZoneTuple = namedtuple( 'ZoneTuple', [ 'name' ] )
-
-    >>> zones = [ ZoneTuple( 'us-west-2a' ), ZoneTuple( 'us-west-2b' ) ]
-    >>> spot_history = [ FauxHistory( 0.1, 'us-west-2a' ), \
-                         FauxHistory( 0.2,'us-west-2a'), \
-                         FauxHistory( 0.3,'us-west-2b'), \
-                         FauxHistory( 0.6,'us-west-2b')]
-    >>> # noinspection PyProtectedMember
-    >>> choose_spot_zone( zones, 0.15, spot_history )
+    >>> FauxHistory = namedtuple('FauxHistory', ['price', 'availability_zone'])
+    >>> ZoneTuple = namedtuple('ZoneTuple', ['name'])
+    >>> zones = [ZoneTuple('us-west-2a'), ZoneTuple('us-west-2b')]
+    >>> spot_history = [FauxHistory(0.1, 'us-west-2a'), \
+                        FauxHistory(0.2, 'us-west-2a'), \
+                        FauxHistory(0.3, 'us-west-2b'), \
+                        FauxHistory(0.6, 'us-west-2b')]
+    >>> choose_spot_zone(zones, 0.15, spot_history)
     'us-west-2a'
 
-    >>> spot_history=[ FauxHistory( 0.3, 'us-west-2a' ), \
-                       FauxHistory( 0.2, 'us-west-2a' ), \
-                       FauxHistory( 0.1, 'us-west-2b'), \
-                       FauxHistory( 0.6, 'us-west-2b') ]
-    >>> # noinspection PyProtectedMember
+    >>> spot_history=[FauxHistory(0.3, 'us-west-2a'), \
+                      FauxHistory(0.2, 'us-west-2a'), \
+                      FauxHistory(0.1, 'us-west-2b'), \
+                      FauxHistory(0.6, 'us-west-2b')]
     >>> choose_spot_zone(zones, 0.15, spot_history)
     'us-west-2b'
 
-    >>> spot_history={ FauxHistory( 0.1, 'us-west-2a' ), \
-                       FauxHistory( 0.7, 'us-west-2a' ), \
-                       FauxHistory( 0.1, "us-west-2b" ), \
-                       FauxHistory( 0.6, 'us-west-2b' ) }
-    >>> # noinspection PyProtectedMember
+    >>> spot_history=[FauxHistory(0.1, 'us-west-2a'), \
+                      FauxHistory(0.7, 'us-west-2a'), \
+                      FauxHistory(0.1, 'us-west-2b'), \
+                      FauxHistory(0.6, 'us-west-2b')]
     >>> choose_spot_zone(zones, 0.15, spot_history)
     'us-west-2b'
-   """
-
-    # Create two lists of tuples of form: [ (zone.name, std_deviation), ... ] one for zones
+    """
+    # Create two lists of tuples of form: [(zone.name, std_deviation), ...] one for zones
     # over the bid price and one for zones under bid price. Each are sorted by increasing
     # standard deviation values.
-    #
     markets_under_bid, markets_over_bid = [], []
     for zone in zones:
         zone_histories = [zone_history for zone_history in spot_history if zone_history.availability_zone == zone.name]
@@ -121,8 +122,7 @@ def choose_spot_zone(zones, bid, spot_history):
         zone_tuple = ZoneTuple(name=zone.name, price_deviation=price_deviation)
         (markets_over_bid, markets_under_bid)[recent_price < bid].append(zone_tuple)
 
-    return min(markets_under_bid or markets_over_bid,
-               key=attrgetter('price_deviation')).name
+    return min(markets_under_bid or markets_over_bid, key=attrgetter('price_deviation')).name
 
 
 def optimize_spot_bid(ctx, instance_type, spot_bid):
@@ -170,6 +170,7 @@ def _check_spot_bid(spot_bid, spot_history):
         logger.warn("Your bid $ %f is more than double this instance type's average "
                  "spot price ($ %f) over the last week", spot_bid, average)
 
+
 def _get_spot_history(ctx, instance_type):
     """
     Returns list of 1,000 most recent spot market data points represented as SpotPriceHistory
@@ -197,6 +198,7 @@ sdbFullPolicy = dict(Version="2012-10-17", Statement=[
 iamFullPolicy = dict(Version="2012-10-17", Statement=[
     dict(Effect="Allow", Resource="*", Action="iam:*")])
 
+
 def checkValidNodeTypes(provisioner, nodeTypes):
     """
     Raises if an invalid nodeType is specified for aws, azure, or gce.
@@ -225,8 +227,14 @@ def checkValidNodeTypes(provisioner, nodeTypes):
             if nodeType and ':' in nodeType:
                 nodeType = nodeType.split(':')[0]
             if nodeType not in regionDict[currentZone]:
-                raise RuntimeError('Invalid nodeType (%s) specified for AWS in region: %s.'
-                                   '' % (nodeType, currentZone))
+                # They probably misspelled it and can't tell.
+                close = get_close_matches(nodeType, regionDict[currentZone], 1)
+                if len(close) > 0:
+                    helpText = ' Did you mean ' + close[0] + '?'
+                else:
+                    helpText = ''
+                raise RuntimeError('Invalid nodeType (%s) specified for AWS in region: %s.%s'
+                                   '' % (nodeType, currentZone, helpText))
     # Only checks if aws nodeType specified for gce/azure atm.
     if provisioner == 'gce' or provisioner == 'azure':
         for nodeType in nodeTypes:
