@@ -25,13 +25,13 @@ from toil.lib.memoize import memoize
 import boto.ec2
 from boto.ec2.blockdevicemapping import BlockDeviceMapping, BlockDeviceType
 from boto.exception import BotoServerError, EC2ResponseError
+from boto.utils import get_instance_metadata
 from toil.lib.ec2 import (a_short_time, create_ondemand_instances,
                           create_spot_instances, wait_instances_running, wait_transition)
 from toil.lib.misc import truncExpBackoff
 from toil.provisioners.abstractProvisioner import AbstractProvisioner, Shape
 from toil.provisioners.aws import *
 from toil.lib.context import Context
-from boto.utils import get_instance_metadata
 from toil.lib.retry import retry
 from toil.lib.memoize import less_strict_bool
 from toil.provisioners import NoSuchClusterException
@@ -40,6 +40,12 @@ from toil.lib.generatedEC2Lists import E2Instances
 
 logger = logging.getLogger(__name__)
 logging.getLogger("boto").setLevel(logging.CRITICAL)
+# Role name (used as the suffix) for EC2 instance profiles that are automatically created by Toil.
+_INSTANCE_PROFILE_ROLE_NAME = 'toil'
+# The tag key that specifies the Toil node type ("leader" or "worker") so that
+# leader vs. worker nodes can be robustly identified.
+_TOIL_NODE_TYPE_TAG_KEY = 'ToilNodeType'
+
 
 def awsRetryPredicate(e):
     if not isinstance(e, BotoServerError):
@@ -95,6 +101,7 @@ class AWSProvisioner(AbstractProvisioner):
 
     def __init__(self, clusterName, zone, nodeStorage, sseKey):
         super(AWSProvisioner, self).__init__(clusterName, zone, nodeStorage)
+        self.cloud = 'aws'
         self._sseKey = sseKey
         if not zone:
             self._zone = getCurrentAWSZone()
@@ -109,7 +116,7 @@ class AWSProvisioner(AbstractProvisioner):
         is the leader.
         """
         instanceMetaData = get_instance_metadata()
-        region = Context.availability_zone_re.match(self._zone).group(1)
+        region = zoneToRegion(self._zone)
         conn = boto.ec2.connect_to_region(region)
         instance = conn.get_all_instances(instance_ids=[instanceMetaData["instance-id"]])[0].instances[0]
         self.clusterName = str(instance.tags["Name"])
@@ -119,6 +126,7 @@ class AWSProvisioner(AbstractProvisioner):
         self._keyName = list(instanceMetaData['public-keys'].keys())[0]
         self._tags = self.getLeader().tags
         self._masterPublicKey = self._setSSH()
+        self._leaderProfileArn = instanceMetaData['iam']['info']['InstanceProfileArn']
 
     def launchCluster(self, leaderNodeType, leaderStorage, owner, **kwargs):
         """
@@ -130,9 +138,9 @@ class AWSProvisioner(AbstractProvisioner):
         if 'keyName' not in kwargs:
             raise RuntimeError("A keyPairName is required for the AWS provisioner.")
         self._keyName = kwargs['keyName']
-        self._vpcSubnet = kwargs['vpcSubnet'] if 'vpcSubnet' in kwargs else None
+        self._vpcSubnet = kwargs.get('vpcSubnet')
 
-        profileARN = self._getProfileARN()
+        profileArn = kwargs.get('awsEc2ProfileArn') or self._getProfileArn()
         # the security group name is used as the cluster identifier
         sgs = self._createSecurityGroup()
         bdm = self._getBlockDeviceMapping(E2Instances[leaderNodeType], rootVolSize=leaderStorage)
@@ -142,7 +150,7 @@ class AWSProvisioner(AbstractProvisioner):
         specKwargs = {'key_name': self._keyName, 'security_group_ids': [sg.id for sg in sgs],
                   'instance_type': leaderNodeType,
                   'user_data': userData, 'block_device_map': bdm,
-                  'instance_profile_arn': profileARN,
+                  'instance_profile_arn': profileArn,
                   'placement': self._zone}
         if self._vpcSubnet:
             specKwargs["subnet_id"] = self._vpcSubnet
@@ -158,7 +166,7 @@ class AWSProvisioner(AbstractProvisioner):
                           preemptable=False, tags=leader.tags)
         leaderNode.waitForNode('toil_leader')
 
-        defaultTags = {'Name': self.clusterName, 'Owner': owner}
+        defaultTags = {'Name': self.clusterName, 'Owner': owner, _TOIL_NODE_TYPE_TAG_KEY: 'leader'}
         if kwargs['userTags']:
             defaultTags.update(kwargs['userTags'])
 
@@ -247,7 +255,6 @@ class AWSProvisioner(AbstractProvisioner):
                 raise RuntimeError("No spot bid given for a preemptable node request.")
         instanceType = E2Instances[nodeType]
         bdm = self._getBlockDeviceMapping(instanceType, rootVolSize=self._nodeStorage)
-        arn = self._getProfileARN()
 
         keyPath = self._sseKey if self._sseKey else None
         userData =  self._getCloudConfigUserData('worker', self._masterPublicKey, keyPath, preemptable)
@@ -257,7 +264,7 @@ class AWSProvisioner(AbstractProvisioner):
                   'instance_type': instanceType.name,
                   'user_data': userData,
                   'block_device_map': bdm,
-                  'instance_profile_arn': arn,
+                  'instance_profile_arn': self._leaderProfileArn,
                   'placement': self._zone,
                   'subnet_id': self._subnetID}
 
@@ -265,7 +272,7 @@ class AWSProvisioner(AbstractProvisioner):
 
         for attempt in retry(predicate=awsRetryPredicate):
             with attempt:
-                # after we start launching instances we want to insure the full setup is done
+                # after we start launching instances we want to ensure the full setup is done
                 # the biggest obstacle is AWS request throttling, so we retry on these errors at
                 # every request in this method
                 if not preemptable:
@@ -291,6 +298,7 @@ class AWSProvisioner(AbstractProvisioner):
             with attempt:
                 wait_instances_running(self._ctx.ec2, instancesLaunched)
 
+        self._tags[_TOIL_NODE_TYPE_TAG_KEY] = 'worker'
         AWSProvisioner._addTags(instancesLaunched, self._tags)
         if self._sseKey:
             for i in instancesLaunched:
@@ -322,7 +330,7 @@ class AWSProvisioner(AbstractProvisioner):
             self._zone = getCurrentAWSZone()
             if self._zone is None:
                 raise RuntimeError(
-                    'Could not determine availability zone. Insure that one of the following '
+                    'Could not determine availability zone. Ensure that one of the following '
                     'is true: the --zone flag is set, the TOIL_AWS_ZONE environment variable '
                     'is set, ec2_region_name is set in the .boto file, or that '
                     'you are running on EC2.')
@@ -332,7 +340,7 @@ class AWSProvisioner(AbstractProvisioner):
     @memoize
     def _discoverAMI(self):
         def descriptionMatches(ami):
-            return ami.description is not None and 'stable 1745.7.0' in ami.description
+            return ami.description is not None and 'stable 1855.5.0' in ami.description
         coreOSAMI = os.environ.get('TOIL_AWS_AMI')
         if coreOSAMI is not None:
             return coreOSAMI
@@ -341,11 +349,11 @@ class AWSProvisioner(AbstractProvisioner):
             # SSLError is thrown when get_all_images times out
             with attempt:
                 # 679593333241 is the aws marketplace account
-                amis = self._ctx.ec2.get_all_images(owners=['679593333241'], filters={'name': 'CoreOS-stable-1745.7.0-hvm-0d1e0bd0-eaea-4397-9a3a-c56f861d2a14-ami-f6ecac89.4'})
+                amis = self._ctx.ec2.get_all_images(owners=['679593333241'], filters={'name': 'CoreOS-stable-1855.5.0-hvm-0d1e0bd0-eaea-4397-9a3a-c56f861d2a14-ami-0f74e41ea6c13f74b.4'})
 
         coreOSAMI = [ami for ami in amis if descriptionMatches(ami)]
         logger.debug('Found the following matching AMIs: %s', coreOSAMI)
-        assert len(coreOSAMI) == 1
+        assert len(coreOSAMI) == 1, coreOSAMI
         return coreOSAMI.pop().id
 
     def _toNameSpace(self):
@@ -366,6 +374,12 @@ class AWSProvisioner(AbstractProvisioner):
             leader = instances[0]  # assume leader was launched first
         except IndexError:
             raise NoSuchClusterException(self.clusterName)
+        if (leader.tags.get(_TOIL_NODE_TYPE_TAG_KEY) or 'leader') != 'leader':
+            raise RuntimeError(
+                'Invalid cluster state! The first launched instance appears not to be the leader '
+                'as it is missing the "leader" tag. The safest recovery is to destroy the cluster '
+                'and restart the job. Incorrect Leader ID: %s' % leader.id
+            )
         leaderNode = Node(publicIP=leader.ip_address, privateIP=leader.private_ip_address,
                           name=leader.id, launchTime=leader.launch_time, nodeType=None,
                           preemptable=False, tags=leader.tags)
@@ -400,7 +414,7 @@ class AWSProvisioner(AbstractProvisioner):
         while True:
             time.sleep(a_short_time)
             instance.update()
-            if instance.ip_address or instance.public_dns_name:
+            if instance.ip_address or instance.public_dns_name or instance.private_ip_address:
                 logger.debug('...got ip')
                 break
 
@@ -426,6 +440,11 @@ class AWSProvisioner(AbstractProvisioner):
             # boto won't look things up by the ARN so we have to parse it to get
             # the profile name
             profileName = profile.rsplit('/')[-1]
+
+            # Only delete profiles that were automatically created by Toil.
+            if profileName != self._ctx.to_aws_name(_INSTANCE_PROFILE_ROLE_NAME):
+                continue
+
             try:
                 profileResult = self._ctx.iam.get_instance_profile(profileName)
             except BotoServerError as e:
@@ -476,7 +495,7 @@ class AWSProvisioner(AbstractProvisioner):
     @classmethod
     def _getBlockDeviceMapping(cls, instanceType, rootVolSize=50):
         # determine number of ephemeral drives via cgcloud-lib (actually this is moved into toil's lib
-        bdtKeys = [''] + ['/dev/xvd{}'.format(c) for c in string.lowercase[1:]]
+        bdtKeys = [''] + ['/dev/xvd{}'.format(c) for c in string.ascii_lowercase[1:]]
         bdm = BlockDeviceMapping()
         # Change root volume size to allow for bigger Docker instances
         root_vol = BlockDeviceType(delete_on_termination=True)
@@ -554,14 +573,13 @@ class AWSProvisioner(AbstractProvisioner):
         return out
 
     @awsRetry
-    def _getProfileARN(self):
+    def _getProfileArn(self):
         assert self._ctx
         def addRoleErrors(e):
             return e.status == 404
-        roleName = 'toil'
         policy = dict(iam_full=iamFullPolicy, ec2_full=ec2FullPolicy,
                       s3_full=s3FullPolicy, sbd_full=sdbFullPolicy)
-        iamRoleName = self._ctx.setup_iam_ec2_role(role_name=roleName, policies=policy)
+        iamRoleName = self._ctx.setup_iam_ec2_role(role_name=_INSTANCE_PROFILE_ROLE_NAME, policies=policy)
 
         try:
             profile = self._ctx.iam.get_instance_profile(iamRoleName)
