@@ -118,6 +118,7 @@ class SortTest(ToilTest, MesosTestSupport, ParasolTestSupport):
                 options.outputFile = self.outputFile
                 options.fileToSort = self.inputFile
                 options.overwriteOutput = True
+                options.realTimeLogging = True
 
                 # Make the file to sort
                 makeFileToSort(options.fileToSort, lines=lines, lineLen=lineLen)
@@ -134,7 +135,7 @@ class SortTest(ToilTest, MesosTestSupport, ParasolTestSupport):
                         # Now check the file is properly sorted..
                         with open(options.outputFile, 'r') as fileHandle:
                             l2 = fileHandle.readlines()
-                            self.assertEquals(l, l2)
+                            self.assertEqual(l, l2)
 
                 options.restart = False
 
@@ -267,7 +268,7 @@ class SortTest(ToilTest, MesosTestSupport, ParasolTestSupport):
             sort(tempFile1)
             with open(tempFile1, 'r') as f:
                 lines2 = f.readlines()
-            self.assertEquals(lines1, lines2)
+            self.assertEqual(lines1, lines2)
 
     def testMerge(self):
         for test in range(self.testNo):
@@ -286,7 +287,7 @@ class SortTest(ToilTest, MesosTestSupport, ParasolTestSupport):
             lines1.sort()
             with open(tempFile3, 'r') as f:
                 lines2 = f.readlines()
-            self.assertEquals(lines1, lines2)
+            self.assertEqual(lines1, lines2)
 
     def testCopySubRangeOfFile(self):
         for test in range(self.testNo):
@@ -297,12 +298,13 @@ class SortTest(ToilTest, MesosTestSupport, ParasolTestSupport):
             assert fileSize > 0
             fileStart = random.choice(range(0, fileSize))
             fileEnd = random.choice(range(fileStart, fileSize))
-            fileHandle = open(outputFile, 'w')
-            copySubRangeOfFile(tempFile, fileStart, fileEnd, fileHandle)
-            fileHandle.close()
-            l = open(outputFile, 'r').read()
-            l2 = open(tempFile, 'r').read()[fileStart:fileEnd]
-            self.assertEquals(l, l2)
+            with open(outputFile, 'w') as f:
+                f.write(copySubRangeOfFile(tempFile, fileStart, fileEnd))
+            with open(outputFile, 'r') as f:
+                l = f.read()
+            with open(tempFile, 'r') as f:
+                l2 = f.read()[fileStart:fileEnd]
+            self.assertEqual(l, l2)
 
     def testGetMidPoint(self):
         for test in range(self.testNo):
