@@ -1337,7 +1337,7 @@ def _exportStaticMethodAsGlobalFunctions(cls):
     the convention that the first argument of a job function is named 'job'.
     """
     for name, kind, clazz, value in inspect.classify_class_attrs(cls):
-        if kind == 'static method':
+        if kind == 'static method' and name != '__new__':  # __new__ became static in 3.7
             method = value.__func__
             args = inspect.getargspec(method).args
             if args and args[0] == 'job':
