@@ -11,7 +11,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 from __future__ import absolute_import, print_function
 from future import standard_library
 standard_library.install_aliases()
@@ -23,7 +22,6 @@ import sys
 import copy
 import random
 import json
-
 import tempfile
 import traceback
 import time
@@ -32,13 +30,6 @@ import logging
 import shutil
 from threading import Thread
 
-logging.basicConfig()
-
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
 from toil.lib.expando import MagicExpando
 from toil.common import Toil, safeUnpickleFromStream
 from toil.fileStore import FileStore
@@ -49,6 +40,7 @@ from toil.lib.bioio import getTotalCpuTime
 from toil.lib.bioio import getTotalCpuTimeAndMemoryUsage
 import signal
 
+logging.basicConfig()
 logger = logging.getLogger(__name__)
 
 def nextChainableJobGraph(jobGraph, jobStore):
@@ -95,7 +87,7 @@ def nextChainableJobGraph(jobGraph, jobStore):
 
     # Somewhat ugly, but check if job is a checkpoint job and quit if
     # so
-    if successorJobGraph.command.startswith( "_toil " ):
+    if successorJobGraph.command.startswith("_toil "):
         #Load the job
         successorJob = Job._loadJob(successorJobGraph.command, jobStore)
 
@@ -126,21 +118,39 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     logFileByteReportLimit = config.maxLogFileSize
 
     if config.badWorker > 0 and random.random() < config.badWorker:
-        def badWorker():
-            #This will randomly kill the worker process at a random time 
-            time.sleep(config.badWorkerFailInterval * random.random())
-            os.kill(os.getpid(), signal.SIGKILL) #signal.SIGINT)
-            #TODO: FIX OCCASIONAL DEADLOCK WITH SIGINT (tested on single machine)
-        t = Thread(target=badWorker)
-        # Ideally this would be a daemon thread but that causes an intermittent (but benign)
-        # exception similar to the one described here:
-        # http://stackoverflow.com/questions/20596918/python-exception-in-thread-thread-1-most-likely-raised-during-interpreter-shutd
-        # Our exception is:
-        #    Exception in thread Thread-1 (most likely raised during interpreter shutdown):
-        #    <type 'exceptions.AttributeError'>: 'NoneType' object has no attribute 'kill'
-        # This attribute error is caused by the call os.kill() and apparently unavoidable with a
-        # daemon
-        t.start()
+        # We need to kill the process we are currently in, to simulate worker
+        # failure. We don't want to just send SIGKILL, because we can't tell
+        # that from a legitimate OOM on our CI runner. We're going to send
+        # SIGUSR1 so our terminations are distinctive, and then SIGKILL if that
+        # didn't stick. We definitely don't want to do this from *within* the
+        # process we are trying to kill, so we fork off. TODO: We can still
+        # leave the killing code running after the main Toil flow is done, but
+        # since it's now in a process instead of a thread, the main Python
+        # process won't wait around for its timeout to expire. I think this is
+        # better than the old thread-based way where all of Toil would wait
+        # around to be killed.
+        
+        killTarget = os.getpid()
+        sleepTime = config.badWorkerFailInterval * random.random()
+        if os.fork() == 0:
+            # We are the child
+            # Let the parent run some amount of time
+            time.sleep(sleepTime)
+            # Kill it gently
+            os.kill(killTarget, signal.SIGUSR1)
+            # Wait for that to stick
+            time.sleep(0.01)
+            try:
+                # Kill it harder. Hope the PID hasn't already been reused.
+                # If we succeeded the first time, this will OSError
+                os.kill(killTarget, signal.SIGKILL)
+            except OSError:
+                pass
+            # Exit without doing any of Toil's cleanup
+            os._exit()
+            
+        # We don't need to reap the child. Either it kills us, or we finish
+        # before it does. Either way, init will have to clean it up for us.
 
     ##########################################
     #Load the environment for the jobGraph
@@ -159,7 +169,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
                 sys.path.append(e)
 
     toilWorkflowDir = Toil.getWorkflowDir(config.workflowID, config.workDir)
-
+    
     ##########################################
     #Setup the temporary directories.
     ##########################################
@@ -185,6 +195,12 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     tempWorkerLogPath = os.path.join(localWorkerTempDir, "worker_log.txt")
 
     if redirectOutputToLogFile:
+        # Announce that we are redirecting logging, and where it will now go.
+        # This is important if we are trying to manually trace a faulty worker invocation.
+        logger.info("Redirecting logging to %s", tempWorkerLogPath)
+        sys.stdout.flush()
+        sys.stderr.flush()
+        
         # Save the original stdout and stderr (by opening new file descriptors
         # to the same files)
         origStdOut = os.dup(1)
@@ -219,6 +235,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
     statsDict.workers.logsToMaster = []
     blockFn = lambda : True
     listOfJobs = [jobName]
+    job = None
     try:
 
         #Put a message at the top of the log, just to make sure it's working.
@@ -293,7 +310,7 @@ def workerScript(jobStore, config, jobName, jobStoreID, redirectOutputToLogFile=
             ##########################################
             
             if jobGraph.command is not None:
-                assert jobGraph.command.startswith( "_toil " )
+                assert jobGraph.command.startswith("_toil ")
                 logger.debug("Got a command to run: %s" % jobGraph.command)
                 #Load the job
                 job = Job._loadJob(jobGraph.command, jobStore)
@@ -494,6 +511,15 @@ def main(argv=None):
     if argv is None:
         argv = sys.argv
 
+    # Do a little argument validation, in case someone tries to run us manually.
+    if len(argv) < 4:
+        if len(argv) < 1:
+            sys.stderr.write("Error: Toil worker invoked without its own name\n")
+            sys.exit(1)
+        else:
+            sys.stderr.write("Error: usage: %s JOB_NAME JOB_STORE_LOCATOR JOB_STORE_ID\n" % argv[0])
+            sys.exit(1)
+
     # Parse input args
     jobName = argv[1]
     jobStoreLocator = argv[2]
@@ -503,16 +529,6 @@ def main(argv=None):
     #Load the jobStore/config file
     ##########################################
 
-    # Try to monkey-patch boto early so that credentials are cached.
-    try:
-        import boto
-    except ImportError:
-        pass
-    else:
-        # boto is installed, monkey patch it now
-        from toil.lib.ec2Credentials import enable_metadata_credential_caching
-        enable_metadata_credential_caching()
-
     jobStore = Toil.resumeJobStore(jobStoreLocator)
     config = jobStore.config
 
